@@ -49,6 +49,7 @@ static void out123_clear_module(out123_handle *ao)
 	ao->drain = NULL;
 	ao->close = NULL;
 	ao->deinit = NULL;
+	ao->enumerate = NULL;
 
 	ao->module = NULL;
 	ao->userptr = NULL;
@@ -147,6 +148,8 @@ static const char *const errstring[] =
 ,	"unknown parameter code"
 ,	"attempt to set read-only parameter"
 ,	"invalid out123 handle"
+,	"operation not supported"
+,	"device enumeration failed"
 };
 
 const char* attribute_align_arg out123_strerror(out123_handle *ao)
@@ -205,6 +208,13 @@ out123_set_buffer(out123_handle *ao, size_t buffer_bytes)
 	return 0;
 }
 
+int attribute_align_arg
+out123_param2( out123_handle *ao, int code
+            , long value, double fvalue, const char *svalue )
+{
+	return out123_param(ao, code, value, fvalue, svalue);
+}
+
 int attribute_align_arg
 out123_param( out123_handle *ao, enum out123_parms code
             , long value, double fvalue, const char *svalue )
@@ -268,6 +278,13 @@ out123_param( out123_handle *ao, enum out123_parms code
 	return ret;
 }
 
+int attribute_align_arg
+out123_getparam2( out123_handle *ao, int code
+               , long *ret_value, double *ret_fvalue, char* *ret_svalue )
+{
+	return out123_getparam(ao, code, ret_value, ret_fvalue, ret_svalue);
+}
+
 int attribute_align_arg
 out123_getparam( out123_handle *ao, enum out123_parms code
                , long *ret_value, double *ret_fvalue, char* *ret_svalue )
@@ -999,7 +1016,6 @@ out123_drivers(out123_handle *ao, char ***names, char ***descr)
 	char **tmpnames;
 	char **tmpdescr;
 	int count;
-	int i;
 
 	if(!ao)
 		return -1;
@@ -1038,22 +1054,109 @@ out123_drivers(out123_handle *ao, char ***names, char ***descr)
 
 	/* Return or free gathered lists of names or descriptions. */
 	if(names)
-		*names = tmpnames;
-	else
 	{
-		for(i=0; i<count; ++i)
-			free(tmpnames[i]);
-		free(tmpnames);
+		*names = tmpnames;
+		tmpnames = NULL;
 	}
 	if(descr)
+	{
 		*descr = tmpdescr;
+		tmpdescr = NULL;
+	}
+	out123_stringlists_free(tmpnames, tmpdescr, count);
+	return count;
+}
+
+struct devlist
+{
+	int count;
+	char **names;
+	char **descr;
+};
+
+static int devlist_add(void *dll, const char *name, const char *descr)
+{
+	struct devlist *dl = (struct devlist*)dll;
+	return dl
+	?	stringlists_add(&(dl->names), &(dl->descr), name, descr, &(dl->count))
+	:	-1;
+}
+
+int out123_devices( out123_handle *ao, const char *driver, char ***names, char ***descr
+,	char **active_driver )
+{
+	int ret = 0;
+	struct devlist dl = {0, NULL, NULL};
+	char *realdrv = NULL;
+	debug("");
+	if(!ao)
+		return -1;
+#ifndef NOXFERMEM
+	if(have_buffer(ao))
+		return out123_seterr(ao, OUT123_NOT_SUPPORTED);
+#endif
+
+	ao->errcode = OUT123_OK;
+	// If the driver is a single word, not a list with commas.
+	// Then don't try to open drivers just to know which we are talking about.
+	if(driver && strchr(driver, ',') == NULL)
+		realdrv = compat_strdup(driver);
 	else
 	{
-		for(i=0; i<count; ++i)
-			free(tmpdescr[i]);
-		free(tmpdescr);
+		mdebug("need to find a driver from: %s", driver ? driver : DEFAULT_OUTPUT_MODULE);
+		if(out123_open(ao, driver, NULL) != OUT123_OK)
+			return out123_seterr(ao, OUT123_BAD_DRIVER);
+		mdebug("deduced driver: %s", ao->driver);
+		realdrv = compat_strdup(ao->driver);
 	}
-	return count;
+	if(realdrv == NULL)
+		return out123_seterr(ao, OUT123_DOOM);
+
+	out123_close(ao);
+
+	if(open_fake_module(ao, realdrv) != OUT123_OK)
+	{
+		ao->module = open_module( "output", realdrv
+		,	modverbose(ao, 0), ao->bindir );
+		/* Open the module, initial check for availability+libraries. */
+		if( !ao->module || !ao->module->init_output
+			|| ao->module->init_output(ao) )
+			ret = out123_seterr(ao, OUT123_BAD_DRIVER);
+	}
+
+	if(!ret && ao->enumerate)
+	{
+		if(!ao->enumerate(ao, devlist_add, &dl))
+		{
+			ret = dl.count;
+			if(names)
+			{
+				*names = dl.names;
+				dl.names = NULL;
+			}
+			if(descr)
+			{
+				*descr = dl.descr;
+				dl.descr = NULL;
+			}
+			if(active_driver)
+			{
+				*active_driver = realdrv;
+				realdrv = NULL;
+			}
+		} else
+			ret = out123_seterr(ao, OUT123_DEV_ENUMERATE);
+		out123_stringlists_free(dl.names, dl.descr, dl.count);
+		if(ao->deinit)
+			ao->deinit(ao);
+	} else if(!ret)
+		ret = out123_seterr(ao, OUT123_NOT_SUPPORTED);
+
+	free(realdrv);
+	if(ao->module)
+		close_module(ao->module, modverbose(ao, 0));
+	out123_clear_module(ao);
+	return ret;
 }
 
 /* We always have ao->driver and ao->device set, also with buffer.
